(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(require('yargs'), require('fs'), require('util'), require('child_process'), require('ora'), require('isomorphic-fetch')) :
  typeof define === 'function' && define.amd ? define(['yargs', 'fs', 'util', 'child_process', 'ora', 'isomorphic-fetch'], factory) :
  (global = global || self, factory(global.yargs, global.fs, global.util, global.child, global.ora, global.isomorphicFetch));
}(this, (function (yargs, fs, util, child, ora, fetch) {
  fs = fs && Object.prototype.hasOwnProperty.call(fs, 'default') ? fs['default'] : fs;
  util = util && Object.prototype.hasOwnProperty.call(util, 'default') ? util['default'] : util;
  child = child && Object.prototype.hasOwnProperty.call(child, 'default') ? child['default'] : child;
  ora = ora && Object.prototype.hasOwnProperty.call(ora, 'default') ? ora['default'] : ora;
  fetch = fetch && Object.prototype.hasOwnProperty.call(fetch, 'default') ? fetch['default'] : fetch;

  const writeFile = util.promisify(fs.writeFile);
  const appendFile = util.promisify(fs.appendFile);
  const exec = util.promisify(child.exec);
  const isTTY = process.stdin.isTTY;
  const LOG_FILE = 'where-broke.log';
  const log = {
    clear: () => writeFile(LOG_FILE, ''),
    log: msg => appendFile(LOG_FILE, msg)
  };

  function _settle(pact, state, value) {
    if (!pact.s) {
      if (value instanceof _Pact) {
        if (value.s) {
          if (state & 1) {
            state = value.s;
          }

          value = value.v;
        } else {
          value.o = _settle.bind(null, pact, state);
          return;
        }
      }

      if (value && value.then) {
        value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
        return;
      }

      pact.s = state;
      pact.v = value;
      const observer = pact.o;

      if (observer) {
        observer(pact);
      }
    }
  }

  const testCandidate = function (candidate) {
    try {
      function _temp2() {
        const accepted = key === 'y';
        process.stdout.clearLine();
        process.stdout.cursorTo(0);

        if (accepted) {
          ora().succeed(`accepted ${candidate}`);
        } else {
          ora().fail(`rejected ${candidate}`);
        }

        return accepted;
      }

      process.stdout.write(`Is '${candidate}' a good candidate? [y/n]: `);
      let key = null;

      const _temp = _for(function () {
        return key !== 'y' && key !== 'n';
      }, void 0, function () {
        return Promise.resolve(keypress()).then(function (_keypress) {
          key = _keypress;
        });
      });

      return Promise.resolve(_temp && _temp.then ? _temp.then(_temp2) : _temp2(_temp));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  const _Pact = /*#__PURE__*/function () {
    function _Pact() {}

    _Pact.prototype.then = function (onFulfilled, onRejected) {
      const result = new _Pact();
      const state = this.s;

      if (state) {
        const callback = state & 1 ? onFulfilled : onRejected;

        if (callback) {
          try {
            _settle(result, 1, callback(this.v));
          } catch (e) {
            _settle(result, 2, e);
          }

          return result;
        } else {
          return this;
        }
      }

      this.o = function (_this) {
        try {
          const value = _this.v;

          if (_this.s & 1) {
            _settle(result, 1, onFulfilled ? onFulfilled(value) : value);
          } else if (onRejected) {
            _settle(result, 1, onRejected(value));
          } else {
            _settle(result, 2, value);
          }
        } catch (e) {
          _settle(result, 2, e);
        }
      };

      return result;
    };

    return _Pact;
  }();

  function _isSettledPact(thenable) {
    return thenable instanceof _Pact && thenable.s & 1;
  }

  function _for(test, update, body) {
    var stage;

    for (;;) {
      var shouldContinue = test();

      if (_isSettledPact(shouldContinue)) {
        shouldContinue = shouldContinue.v;
      }

      if (!shouldContinue) {
        return result;
      }

      if (shouldContinue.then) {
        stage = 0;
        break;
      }

      var result = body();

      if (result && result.then) {
        if (_isSettledPact(result)) {
          result = result.s;
        } else {
          stage = 1;
          break;
        }
      }

      if (update) {
        var updateValue = update();

        if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
          stage = 2;
          break;
        }
      }
    }

    var pact = new _Pact();

    var reject = _settle.bind(null, pact, 2);

    (stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
    return pact;

    function _resumeAfterBody(value) {
      result = value;

      do {
        if (update) {
          updateValue = update();

          if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
            updateValue.then(_resumeAfterUpdate).then(void 0, reject);
            return;
          }
        }

        shouldContinue = test();

        if (!shouldContinue || _isSettledPact(shouldContinue) && !shouldContinue.v) {
          _settle(pact, 1, result);

          return;
        }

        if (shouldContinue.then) {
          shouldContinue.then(_resumeAfterTest).then(void 0, reject);
          return;
        }

        result = body();

        if (_isSettledPact(result)) {
          result = result.v;
        }
      } while (!result || !result.then);

      result.then(_resumeAfterBody).then(void 0, reject);
    }

    function _resumeAfterTest(shouldContinue) {
      if (shouldContinue) {
        result = body();

        if (result && result.then) {
          result.then(_resumeAfterBody).then(void 0, reject);
        } else {
          _resumeAfterBody(result);
        }
      } else {
        _settle(pact, 1, result);
      }
    }

    function _resumeAfterUpdate() {
      if (shouldContinue = test()) {
        if (shouldContinue.then) {
          shouldContinue.then(_resumeAfterTest).then(void 0, reject);
        } else {
          _resumeAfterTest(shouldContinue);
        }
      } else {
        _settle(pact, 1, result);
      }
    }
  }

  const keypress = function () {
    try {
      return Promise.resolve(new Promise(resolve => ttys.stdin.once('data', key => {
        resolve(key);
      })));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  const getCandidates = function () {
    try {
      if (isTTY) {
        throw new Error('manual mode currently only works when providing candidates via stdin');
      }

      const input = fs.readFileSync(0, 'utf-8');
      return Promise.resolve(input.replace(/\r/g, '').split('\n').map(x => x.trim()).filter(Boolean));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  const afterAll = function () {
    try {
      ttys.stdin.destroy();
      return Promise.resolve();
    } catch (e) {
      return Promise.reject(e);
    }
  };

  const beforeAll = function () {
    try {
      ttys = require('ttys');
      ttys.stdin.setRawMode(true);
      ttys.stdin.setEncoding('utf8');
      return Promise.resolve();
    } catch (e) {
      return Promise.reject(e);
    }
  };

  let ttys = null;
  var manual = {
    getCandidates,
    testCandidate,
    beforeAll,
    afterAll
  };

  function _catch(body, recover) {
    try {
      var result = body();
    } catch (e) {
      return recover(e);
    }

    if (result && result.then) {
      return result.then(void 0, recover);
    }

    return result;
  }

  const testCandidate$1 = function (candidate, attempt, {
    module
  }) {
    try {
      const name = `${module}@${candidate}`;
      const spinner = ora(`install  ${name}`).start();
      return Promise.resolve(exec(`npm install ${name} --no-save --no-audit`)).then(function () {
        return _catch(function () {
          spinner.text = `testing  ${name}`;
          return Promise.resolve(exec('npm run test')).then(function () {
            spinner.succeed(`accepted ${name}`);
            return true;
          });
        }, function (ex) {
          return Promise.resolve(log.log(ex.stderr + '\n\n' + '-'.repeat(70) + '\n\n')).then(function () {
            spinner.fail(`rejected ${name}`);
            return false;
          });
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  const getCandidates$1 = function ({
    module
  }) {
    try {
      console.log(`Fetch version data from registry.npmjs.org`);
      return Promise.resolve(fetch(`https://registry.npmjs.org/${module}`)).then(function (response) {
        return Promise.resolve(response.json()).then(function (data) {
          return Object.keys(data.versions || {});
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var npm = {
    getCandidates: getCandidates$1,
    testCandidate: testCandidate$1
  };

  function _settle$1(pact, state, value) {
    if (!pact.s) {
      if (value instanceof _Pact$1) {
        if (value.s) {
          if (state & 1) {
            state = value.s;
          }

          value = value.v;
        } else {
          value.o = _settle$1.bind(null, pact, state);
          return;
        }
      }

      if (value && value.then) {
        value.then(_settle$1.bind(null, pact, state), _settle$1.bind(null, pact, 2));
        return;
      }

      pact.s = state;
      pact.v = value;
      const observer = pact.o;

      if (observer) {
        observer(pact);
      }
    }
  }

  const _Pact$1 = /*#__PURE__*/function () {
    function _Pact() {}

    _Pact.prototype.then = function (onFulfilled, onRejected) {
      const result = new _Pact();
      const state = this.s;

      if (state) {
        const callback = state & 1 ? onFulfilled : onRejected;

        if (callback) {
          try {
            _settle$1(result, 1, callback(this.v));
          } catch (e) {
            _settle$1(result, 2, e);
          }

          return result;
        } else {
          return this;
        }
      }

      this.o = function (_this) {
        try {
          const value = _this.v;

          if (_this.s & 1) {
            _settle$1(result, 1, onFulfilled ? onFulfilled(value) : value);
          } else if (onRejected) {
            _settle$1(result, 1, onRejected(value));
          } else {
            _settle$1(result, 2, value);
          }
        } catch (e) {
          _settle$1(result, 2, e);
        }
      };

      return result;
    };

    return _Pact;
  }();

  function _isSettledPact$1(thenable) {
    return thenable instanceof _Pact$1 && thenable.s & 1;
  }

  function _for$1(test, update, body) {
    var stage;

    for (;;) {
      var shouldContinue = test();

      if (_isSettledPact$1(shouldContinue)) {
        shouldContinue = shouldContinue.v;
      }

      if (!shouldContinue) {
        return result;
      }

      if (shouldContinue.then) {
        stage = 0;
        break;
      }

      var result = body();

      if (result && result.then) {
        if (_isSettledPact$1(result)) {
          result = result.s;
        } else {
          stage = 1;
          break;
        }
      }

      if (update) {
        var updateValue = update();

        if (updateValue && updateValue.then && !_isSettledPact$1(updateValue)) {
          stage = 2;
          break;
        }
      }
    }

    var pact = new _Pact$1();

    var reject = _settle$1.bind(null, pact, 2);

    (stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
    return pact;

    function _resumeAfterBody(value) {
      result = value;

      do {
        if (update) {
          updateValue = update();

          if (updateValue && updateValue.then && !_isSettledPact$1(updateValue)) {
            updateValue.then(_resumeAfterUpdate).then(void 0, reject);
            return;
          }
        }

        shouldContinue = test();

        if (!shouldContinue || _isSettledPact$1(shouldContinue) && !shouldContinue.v) {
          _settle$1(pact, 1, result);

          return;
        }

        if (shouldContinue.then) {
          shouldContinue.then(_resumeAfterTest).then(void 0, reject);
          return;
        }

        result = body();

        if (_isSettledPact$1(result)) {
          result = result.v;
        }
      } while (!result || !result.then);

      result.then(_resumeAfterBody).then(void 0, reject);
    }

    function _resumeAfterTest(shouldContinue) {
      if (shouldContinue) {
        result = body();

        if (result && result.then) {
          result.then(_resumeAfterBody).then(void 0, reject);
        } else {
          _resumeAfterBody(result);
        }
      } else {
        _settle$1(pact, 1, result);
      }
    }

    function _resumeAfterUpdate() {
      if (shouldContinue = test()) {
        if (shouldContinue.then) {
          shouldContinue.then(_resumeAfterTest).then(void 0, reject);
        } else {
          _resumeAfterTest(shouldContinue);
        }
      } else {
        _settle$1(pact, 1, result);
      }
    }
  }

  const bisect = function (fn, entries, arg) {
    try {
      function _temp2() {
        return {
          lastGood: entries[to],
          firstBad: entries[from]
        };
      }

      let from = 0;
      let to = entries.length - 1;
      let attempt = 0;

      const _temp = _for$1(function () {
        return from <= to;
      }, void 0, function () {
        attempt += 1;
        const mid = Math.floor((from + (to + 1)) / 2);
        const entry = entries[mid];
        return Promise.resolve(Promise.resolve(fn(entry, attempt, arg))).then(function (passed) {
          if (passed) {
            from = mid + 1;
          } else {
            to = mid - 1;
          }
        });
      });

      return Promise.resolve(_temp && _temp.then ? _temp.then(_temp2) : _temp2(_temp));
    } catch (e) {
      return Promise.reject(e);
    }
  };
  function estimate(entries) {
    return Math.ceil(Math.log2(entries.length));
  }

  const main = function (args) {
    try {
      function _temp3() {
        return Promise.resolve(handler.getCandidates(args)).then(function (candidates) {
          if (candidates.length === 0) {
            console.error('No candidates found to bisect');
            process.exit(1);
            return;
          }

          const iterations = estimate(candidates);
          console.log(`Got ${candidates.length} candidates, will need to check ~ ${iterations} of them\n`);
          return Promise.resolve(log.clear()).then(function () {
            return Promise.resolve(bisect(handler.testCandidate, candidates, args)).then(function ({
              lastGood,
              firstBad
            }) {
              if (!lastGood) {
                console.log(`\nAll candidates failed`);
              } else if (!firstBad) {
                console.log(`\nAll candidates passed`);
              } else {
                console.log(`\nLast good: ${lastGood}\nFirst bad: ${firstBad}`);
              }

              const _temp = function () {
                if (typeof handler.afterAll === 'function') {
                  return Promise.resolve(handler.afterAll()).then(function () {});
                }
              }();

              if (_temp && _temp.then) return _temp.then(function () {});
            });
          });
        });
      }

      const {
        module,
        interactive
      } = args;
      console.log([`Check for regression`, module && `in ${module}`].filter(Boolean).join(' '));
      let handler;

      if (interactive) {
        handler = manual;
      } else if (args.module) {
        handler = npm;
      }

      const _temp2 = function () {
        if (typeof handler.beforeAll === 'function') {
          return Promise.resolve(handler.beforeAll()).then(function () {});
        }
      }();

      return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  yargs.scriptName('where-broke').usage('$0 <cmd> [args]').command('$0 [module]', 'find the version of <module> that broke your tests', {
    module: {
      alias: 'm',
      type: 'string',
      describe: 'the module that we are bisecting',
      demand: isTTY,
      positional: true
    },
    interactive: {
      alias: 'i',
      type: 'boolean',
      describe: 'the values to iterate over, comma separated',
      default: !isTTY
    }
  }, main).help().argv;

})));
//# sourceMappingURL=index.js.map
